﻿using System;
using System.ServiceModel.Channels;
using System.ServiceModel.Dispatcher;


namespace FC12SamlExample
{
    // Class adds authentication headers to standard to soap messages
    public class HttpAuthHeaderMessageInspector : IClientMessageInspector
    {
        private const string AuthTicketName = "AuthTicket";

        public string AuthTicket { get; private set; }

        public HttpAuthHeaderMessageInspector( string authTicket )
        {
            AuthTicket = authTicket;
        }
        
        public object BeforeSendRequest( ref System.ServiceModel.Channels.Message request, System.ServiceModel.IClientChannel channel )
        {
            const string AuthorizationHeaderName = "Authorization";
            const string BearerPrefix = "Bearer: ";

            if ( request.Properties.TryGetValue( HttpRequestMessageProperty.Name, out object httpRequestMessageObject ) )
            {
                var httpRequestMessage = (HttpRequestMessageProperty)httpRequestMessageObject;

                httpRequestMessage.Headers[AuthorizationHeaderName] = BearerPrefix + AuthTicket;
            }
            else
            {
                var httpRequestMessage = new HttpRequestMessageProperty();

                httpRequestMessage.Headers.Add( AuthorizationHeaderName, BearerPrefix + AuthTicket );

                request.Properties.Add( HttpRequestMessageProperty.Name, httpRequestMessage );
            }

            return null;
        }

        // Refresh current authentication ticket value from response message header
        public void AfterReceiveReply( ref System.ServiceModel.Channels.Message reply, object correlationState )
        {
            if ( reply.Properties.TryGetValue( HttpResponseMessageProperty.Name, out object httpResponseMessageObject ) )
            {
                var httpResponseMessage = (HttpResponseMessageProperty)httpResponseMessageObject;

                if ( !string.IsNullOrEmpty( httpResponseMessage.Headers[AuthTicketName] ) )
                {
                    AuthTicket = httpResponseMessage.Headers[AuthTicketName];
                }
            }
        }
    }
}